#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]

pub mod components;
pub mod constants;
pub mod levels;
pub mod resources;
mod sidebar;
mod state;
pub mod systems;
pub mod tile_data;
pub mod vga_color;

use bracket_lib::prelude::*;
use components::{Monster, Player, Position, Renderable};
use resources::{Clock, LevelNumber, Map, ShowDebugInfo, SoundEffects, SoundOutput, Stats};
use specs::prelude::*;
use state::State;
use std::time::Instant;

fn main() -> BError {
    let context = BTermBuilder::vga(80, 25)
        // .with_tile_dimensions(8, 16)
        .with_fps_cap(60.0)
        .with_title("Kroz")
        .build()?;

    let mut ss = SoundOutput::new();
    let sound_effects = SoundEffects::new(&ss);
    ss.play_sound(sound_effects.startup.clone());

    let mut ecs = World::new();

    ecs.insert(ss);
    ecs.insert(LevelNumber(0));
    ecs.insert(ShowDebugInfo(false));
    ecs.insert(Clock {
        last_ticked: Instant::now(),
        has_ticked: false,
        ticks: 0,
    });
    ecs.insert(sound_effects);
    ecs.insert(Stats {
        score: 1290,
        gems: 14,
        whips: 7,
        teleports: 0,
        keys: 0,
    });

    ecs.register::<Position>();
    ecs.register::<Renderable>();
    ecs.register::<Monster>();
    ecs.register::<Player>();

    let map = Map::from_level(levels::get_level(0));
    map.spawn_entities(&mut ecs);
    ecs.insert(map);

    // let descent_sounds: Vec<Sound> = (20..100)
    //     .rev()
    //     .flat_map(|x| {
    //         (1..10).rev().flat_map(move |y| {
    //             vec![Sound {
    //                 sound_type: SoundType::Tone(x * y * y),
    //                 duration: Duration::from_millis((y as f64 / 1.5) as u64),
    //             }]
    //         })
    //     })
    //     .collect();

    // let descent_effect = gs.sound_system.render_sound_effect(SoundEffect {
    //     sounds: descent_sounds,
    // });

    // let _ = gs.sound_system.play_sound(descent_effect);

    // let effect = gs.sound_system.render_sound_effect(SoundEffect {
    //     sounds: vec![Sound {
    //         sound_type: SoundType::Tone(3500),
    //         duration: Duration::from_millis(4000),
    //     }],
    // });

    // let _ = gs.sound_system.play_sound(effect);

    main_loop(context, State::new(ecs))
}
